<?php
/**
 * Plugin Name: WSJ Recent Comments
 * Plugin URI: https://liangchicun.com/
 * Description: Customize the number of comments and characters for comments in the widget.
 * Author: John Wong
 * Author URI: https://liangchicun.com
 * Version: 1.1.1
 * Stable Tag: 1.1.1
 * Text Domain: wsj-recent-comments
 *
 * @package wsj-recent-comments
 * @author John Wong
 */ 

defined( 'ABSPATH' ) || exit;

class WSJ_Recent_Comments extends WP_Widget {

    public function __construct() {
        parent::__construct(
            'recent_comments_widget',
            __( 'WSJ Recent Comments', 'wsj-recent-comments' ),
            array(
                'description' => __( 'Customize the number of comments and characters for comments in the widget', 'wsj-recent-comments' ),
            )
        );

        add_action( 'comment_post', array( $this, 'flush_widget_cache' ) );
        add_action( 'transition_comment_status', array( $this, 'flush_widget_cache' ) );
    }

    public function flush_widget_cache() {
        wp_cache_delete( 'recent_comments', 'widget' );
    }

    public function widget( $args, $instance ) {
        global $wpdb, $comments, $comment;

        $number = empty( $instance['number'] ) || !is_numeric( $instance['number'] )  ? 5 : absint( $instance['number'] );

        extract( $args, EXTR_SKIP );
        $title = apply_filters( 'widget_title', empty( $instance['title'] ) ? __( 'Recent Comments', 'wsj-recent-comments' ) : $instance['title'] );

        if ( ! $comments = wp_cache_get( 'recent_comments', 'widget' ) ) {
                           $args = array(
                               'status' => 'approve',
                               'post_status' => 'publish',
                               'number' => $number,
                               'order' => 'DESC',
                               'orderby' => 'comment_date_gmt'
                           );
            $comments = get_comments( $args );
            wp_cache_add( 'recent_comments', $comments, 'widget' );
        }

        $excerpt_len = empty( $instance['length'] )  || !is_numeric( $instance['length'] ) ? 50 : absint( $instance['length'] );

        echo wp_kses_post($before_widget);

        if ( $title ) {
            echo wp_kses_post($before_title) . wp_kses_post( $title ) . wp_kses_post($after_title);
        }

         echo '<ul id="recentcomments">';

        if ( $comments ) {
            foreach ( (array) $comments as $comment ) {
                $recent_comment  = get_comment( $comment->comment_ID );
                $recent_comment_text = trim( mb_substr( wp_strip_all_tags( apply_filters( 'comment_text', $recent_comment->comment_content ?? '' ) ), 0, $excerpt_len ) );
                if ( mb_strlen( $recent_comment->comment_content ?? '' ) > $excerpt_len ) {
                    $recent_comment_text .= '...';
                }

                printf(
                    '<li class="recentcomments"><span class="recentcommentsauthor">%s</span> at %s reply<a href="%s"> %s </a>said: <a href="%s">%s</a></li>',
                    get_comment_author_link(),
                    esc_html($comment->comment_date),
                    esc_url(get_permalink($comment->comment_post_ID)),
                    esc_html(get_the_title($comment->comment_post_ID)),
                    esc_url(esc_url(get_comment_link($comment->comment_ID))),
                    esc_html($recent_comment_text)
                );
            }
        }

        echo '</ul>' . wp_kses_post($after_widget);

 }

    public function update( $new_instance, $old_instance ) {
        $instance           = $old_instance;
        $instance['title']  = wp_strip_all_tags( $new_instance['title'] );
        $instance['number'] = (int) $new_instance['number'];
        $instance['length'] = (int) $new_instance['length'];
        $this->flush_widget_cache();

        $alloptions = wp_cache_get( 'alloptions', 'options' );
        if ( isset( $alloptions['widget_recent_comments'] ) ) {
            delete_option( 'widget_recent_comments' );
        }

        return $instance;
    }

    public function form( $instance ) {
        $title  = isset( $instance['title'] ) ? esc_attr( $instance['title'] ) : '';
        $number = empty( $instance['number'] ) ? 5 : absint( $instance['number'] );
        $length = empty( $instance['length'] ) ? 50 : absint( $instance['length'] );

        printf(
            '<p>
                <label for="%s">%s</label>
                <input class="widefat" id="%s" name="%s" type="text" value="%s" />
            </p>',
            esc_attr( $this->get_field_id( 'title' ) ),
            esc_html__( 'Title:', 'wsj-recent-comments' ),
            esc_attr( $this->get_field_id( 'title' ) ),
            esc_attr( $this->get_field_name( 'title' ) ),
            esc_attr( $title )
        );

        printf(
            '<p>
                <label for="%s">%s</label>
                <input id="%s" name="%s" type="text" value="%s" size="3" />
            </p>',
            esc_attr( $this->get_field_id( 'number' ) ),
            esc_html__( 'Number of comments to show:', 'wsj-recent-comments' ),
            esc_attr( $this->get_field_id( 'number' ) ),
            esc_attr( $this->get_field_name( 'number' ) ),
            esc_attr( $number )
        );

        printf(
            '<p>
                <label for="%s">%s</label>
                <input id="%s" name="%s" type="text" value="%s" />
            </p>',
            esc_attr( $this->get_field_id( 'length' ) ),
            esc_html__( 'Comment excerpt character count:', 'wsj-recent-comments' ),
            esc_attr( $this->get_field_id( 'length' ) ),
            esc_attr( $this->get_field_name( 'length' ) ),
            esc_attr( $length )
        );
    }
}

function WSJ_Widget_Recent_Comments_Init() {
    register_widget( 'WSJ_Recent_Comments' );
}

function wsj_recent_comments_enqueue_styles() {
    wp_enqueue_style( 'recent_comments_style', plugin_dir_url( __FILE__ ) . 'style.css', array(), '1.0.0' );
}

add_action( 'widgets_init', 'WSJ_Widget_Recent_Comments_Init' );
add_action( 'wp_enqueue_scripts', 'wsj_recent_comments_enqueue_styles' );
